//+------------------------------------------------------------------+
//|                                         StochasticCyberCycle.mq4 |
//|                                                                  |
//| Stochastic Cyber Cycle                                           |
//|                                                                  |
//| Algorithm taken from book                                        |
//|     "Cybernetics Analysis for Stock and Futures"                 |
//| by John F. Ehlers                                                |
//|                                                                  |
//|                                              contact@mqlsoft.com |
//|                                          http://www.mqlsoft.com/ |
//+------------------------------------------------------------------+
#property copyright "Coded by Witold Wozniak (www.mqlsoft.com)"
#property link      "http://www.mqlsoft.com"
#property version "2.0"
#property description "The algorithm was taken from the book "
#property description "\"Cybernetics Analysis for Stock and Futures\" "
#property description "by John F. Ehlers"
#property strict

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 clrRed
#property indicator_color2 clrBlue

#property indicator_level1 0
#property indicator_minimum -1
#property indicator_maximum 1

double StocCC[];
double Trigger[];
double Smooth[];
double Cycle[];
double Value1[];

input double Alpha = 0.07;
input int Length = 8;
int buffers = 0;
int drawBegin = 8;

int init() {
    drawBegin = Length;
    IndicatorBuffers(5);
    initBuffer(StocCC, "Stochastic Cyber Cycle Open", DRAW_LINE);
    initBuffer(Trigger, "Trigger", DRAW_LINE);
    initBuffer(Smooth);
    initBuffer(Cycle);
    initBuffer(Value1);
    IndicatorShortName("Stochastic Cyber Cycle Open[" + DoubleToStr(Alpha, 2) + ", " + IntegerToString(Length) + "]");
    return (0);
}

int start() {
    if (Bars <= drawBegin) return (0);
    int countedBars = IndicatorCounted();
    if (countedBars < 0) return (-1);
    if (countedBars > 0) countedBars--;
    int s, limit = MathMin(Bars - countedBars - 1, Bars - drawBegin);
    for (s = limit; s >= 0; s--) {
        Smooth[s] = (P(s) + 2.0 * P(s + 1) + 2.0 * P(s + 2) + P(s + 3)) / 6.0;
        Cycle[s] = (1.0 - 0.5 * Alpha) * (1.0 - 0.5 * Alpha) * (Smooth[s] - 2.0 * Smooth[s + 1] + Smooth[s + 2])
            + 2.0 * (1.0 - Alpha) * Cycle[s + 1]
            - (1.0 - Alpha) * (1.0 - Alpha) * Cycle[s + 2];
        if (s > Bars - drawBegin - 8) {
            Cycle[s] = (P(s) - 2.0 * P(s + 1) + P(s + 2)) / 4.0;
        }
        double hh = Cycle[s], ll = Cycle[s];
        for (int i = 0; i < Length; i++) {
            double tmp = Cycle[s + i];
            hh = MathMax(hh, tmp);
            ll = MathMin(ll, tmp);
        }
        Value1[s] = 0.0;
        if (hh != ll) {
            Value1[s] = (Cycle[s] - ll) / (hh - ll);
        }
        StocCC[s] = (4.0 * Value1[s] + 3.0 * Value1[s + 1] + 2.0 * Value1[s + 2] + Value1[s + 3]) / 10.0;
        StocCC[s] = 2.0 * (StocCC[s] - 0.5);
        Trigger[s] = 0.96 * (StocCC[s + 1] + 0.02);
    }
    return (0);   
}

double P(int index) {
    if (index >= Bars) {
        return ((Open[Bars - 1] + Open[Bars - 1]) / 2.0);
    }
    return ((Open[index] + Open[index]) / 2.0);
}

void initBuffer(double& array[], string label = "", int type = DRAW_NONE, int arrow = 0, int style = EMPTY, int width = EMPTY, color clr = CLR_NONE) {
    SetIndexBuffer(buffers, array);
    SetIndexLabel(buffers, label);
    SetIndexEmptyValue(buffers, EMPTY_VALUE);
    SetIndexDrawBegin(buffers, drawBegin);
    SetIndexShift(buffers, 0);
    SetIndexStyle(buffers, type, style, width);
    SetIndexArrow(buffers, arrow);
    buffers++;
}